package io.uqudo.uqudo_flutter_plugin

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.uqudo.sdk.core.*
import io.uqudo.sdk.core.UqudoBuilder.AccountRecovery
import io.uqudo.sdk.core.UqudoBuilder.Enrollment
import io.uqudo.sdk.core.builder.BackgroundCheckConfigurationBuilder
import io.uqudo.sdk.core.builder.FacialRecognitionConfigurationBuilder
import io.uqudo.sdk.core.builder.ReadingConfigurationBuilder
import io.uqudo.sdk.core.domain.model.DocumentType
import io.uqudo.sdk.core.network.ErrorResponseCode
import org.json.JSONObject

/** UqudoFlutterPlugin */
const val METHOD_INIT = "init"
const val METHOD_ENROLL = "enroll"
const val METHOD_RECOVER = "recover"

const val REQUEST_CODE_ENROLL = 101
const val REQUEST_CODE_RECOVER = 102

class UqudoFlutterPlugin: FlutterPlugin, MethodCallHandler, PluginRegistry.ActivityResultListener, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private var context: Context? = null
    private var activity: Activity? = null
    private var pendingResult: Result? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        setContext(flutterPluginBinding.applicationContext)
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().dartExecutor, "uqudo_flutter_plugin")
        channel.setMethodCallHandler(this)
    }


    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val plugin = UqudoFlutterPlugin()
            plugin.setActivity(registrar.activity())
            val channel = MethodChannel(registrar.messenger(), "uqudo_flutter_plugin")
            channel.setMethodCallHandler(plugin)
            registrar.addActivityResultListener(plugin)
        }
    }

    fun setContext(context: Context) {
        this.context = context
    }

    fun setActivity(activity: Activity) {
        this.activity = activity
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            METHOD_INIT -> UqudoSDK.init(context!!)
            METHOD_ENROLL -> {
                pendingResult = result
                enroll(call.argument("message"))
            }
            METHOD_RECOVER -> {
                pendingResult = result
                recover(call.argument("message"))
            }
            else -> result.notImplemented()
        }
    }

    fun enroll(message: String?) {
        kotlin.io.println("message = $message")
        if (message != null && message.isNotEmpty()) {
            try {
                val json = JSONObject(message)
                val enrollment = Enrollment()
                if (json.has("isSecuredWindowsDisabled") && json.getBoolean("isSecuredWindowsDisabled")) {
                    enrollment.disableSecureWindow()
                }
                if (json.has("isRootedDeviceAllowed") && json.getBoolean("isRootedDeviceAllowed")) {
                    enrollment.enableRootedDeviceUsage()
                }
                if (json.has("nonce")) {
                    enrollment.setNonce(json.getString("nonce"))
                }
                if (json.has("sessionId")) {
                    enrollment.setSessionId(json.getString("sessionId"))
                }
                if (json.has("facialRecognitionSpecification")) {
                    val faceBuilder = FacialRecognitionConfigurationBuilder()
                    val faceObject = json.getJSONObject("facialRecognitionSpecification")
                    if (faceObject.has("isHelpPageDisabled") && faceObject.getBoolean("isHelpPageDisabled")) {
                        faceBuilder.disableHelpPage()
                    }
                    if (faceObject.has("enrollFace") && faceObject.getBoolean("enrollFace")) {
                        faceBuilder.enrollFace()
                    }
                    if (faceObject.has("scanMinimumMatchLevel") && faceObject.getInt("scanMinimumMatchLevel") > 0) {
                        faceBuilder.setScanMinimumMatchLevel(faceObject.getInt("scanMinimumMatchLevel"))
                    }
                    if (faceObject.has("readMinimumMatchLevel") && faceObject.getInt("readMinimumMatchLevel") > 0) {
                        faceBuilder.setReadMinimumMatchLevel(faceObject.getInt("readMinimumMatchLevel"))
                    }
                    enrollment.enableFacialRecognition(faceBuilder.build())
                }
                if (json.has("backgroundCheckConfiguration")) {
                    val backgroundCheckConfigurationBuilder = BackgroundCheckConfigurationBuilder()
                    val backgroundObject = json.getJSONObject("backgroundCheckConfiguration")
                    if (backgroundObject.has("disableConsent")) {
                        backgroundCheckConfigurationBuilder.disableConsent()
                    }
                    enrollment.enableBackgroundCheck(backgroundCheckConfigurationBuilder.build())
                }
                enrollment.setToken(json.getString("authorizationToken"))
                val documentList = json.getJSONArray("documentList")
                for (i in 0 until documentList.length()) {
                    val documentJson = documentList.getJSONObject(i)
                    val documentBuilder = DocumentBuilder(context!!)
                    documentBuilder.setDocumentType(DocumentType.valueOf(documentJson.getString("documentType")))
                    if (documentJson.has("isHelpPageVisible") && !documentJson.getBoolean("isHelpPageVisible")) {
                        documentBuilder.disableHelpPage()
                    }
                    if (documentJson.has("readingConfiguration")) {
                        val readConfiguration = documentJson.getJSONObject("readingConfiguration")
                        if (readConfiguration.has("forceReading")) {
                            val readingConfigurationBuilder = ReadingConfigurationBuilder()
                            if (readConfiguration.has("forceReading")) {
                                readingConfigurationBuilder.forceReading(readConfiguration.getBoolean("forceReading"))
                            }
                            if (readConfiguration.has("forceReadingIfSupported")) {
                                readingConfigurationBuilder.forceReadingIfSupported(readConfiguration.getBoolean("forceReadingIfSupported"))
                            }
                            documentBuilder.enableReading(readingConfigurationBuilder.build())
                        } else {
                            documentBuilder.enableReading()
                        }
                    }
                    if (documentJson.has("faceScanMinimumMatchLevel") && documentJson.getInt("faceScanMinimumMatchLevel") > 0) {
                        documentBuilder.setFaceScanMinimumMatchLevel(documentJson.getInt("faceScanMinimumMatchLevel"))
                    }
                    if (documentJson.has("faceReadMinimumMatchLevel") && documentJson.getInt("faceReadMinimumMatchLevel") > 0) {
                        documentBuilder.setFaceReadMinimumMatchLevel(documentJson.getInt("faceReadMinimumMatchLevel"))
                    }
                    val document = documentBuilder.build()
                    enrollment.add(document)
                }
                val intent = enrollment.build(context!!)
                activity?.startActivityForResult(intent, REQUEST_CODE_ENROLL)
            } catch (e: Exception) {
                Log.d("UqudoPlugin", e.message, e)
                sendError(SessionStatusCode.UNEXPECTED_ERROR.name, e.message, null)
            }
        } else {
            sendError(SessionStatusCode.UNEXPECTED_ERROR.name, "Expected enrollment object as argument.", null)
        }
    }

    fun recover(message: String?) {
        if (message != null && message.isNotEmpty()) {
            try {
                val json = JSONObject(message)
                val recovery = AccountRecovery()
                recovery.setToken(json.getString("token"))
                recovery.setEnrollmentIdentifier(json.getString("enrollmentIdentifier"))
                if (json.has("nonce")) {
                    recovery.setNonce(json.getString("nonce"))
                }
                if (json.has("isRootedDeviceAllowed")) {
                    recovery.enableRootedDeviceUsage()
                }
                if (json.has("isSecuredWindowsDisabled")) {
                    recovery.disableSecureWindow()
                }
                if (json.has("isHelpPageDisabled")) {
                    recovery.disableHelpPage()
                }
                if (json.has("minimumMatchLevel") && json.getInt("minimumMatchLevel") > 0) {
                    recovery.setMinimumMatchLevel(json.getInt("minimumMatchLevel"))
                }
                val intent = recovery.build(context!!)
                activity?.startActivityForResult(intent, REQUEST_CODE_RECOVER)
            } catch (e: Exception) {
                Log.d("UqudoPlugin", e.message, e)
                sendError(SessionStatusCode.UNEXPECTED_ERROR.name, e.message, null)
            }
        } else {
            sendError(SessionStatusCode.UNEXPECTED_ERROR.name, "Expected account recovery object as argument.", null)
        }
    }

    private fun sendError(code: String, message: String?, task: String?) {
        val error = JSONObject()
        try {
            error.put("code", code)
            error.put("message", message)
            error.put("task", task)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        pendingResult?.error(error.toString(), null, null)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        if (requestCode == REQUEST_CODE_ENROLL) {
            if (resultCode == Activity.RESULT_OK) {
                pendingResult?.success(data!!.getStringExtra("data"))
            } else if (resultCode == Activity.RESULT_CANCELED) {
                if (data != null) {
                    //Something wrong happened while using the SDK
                    val sessionStatus: SessionStatus = data.getParcelableExtra("key_session_status")!!
                    sendError(sessionStatus.sessionStatusCode.name, sessionStatus.sessionStatusCode.message, sessionStatus.sessionTask.name)
                }
            }
        } else if (requestCode == REQUEST_CODE_RECOVER) {
            if (resultCode == Activity.RESULT_OK) {
                pendingResult?.success(data!!.getStringExtra("data"))
            } else if (resultCode == Activity.RESULT_CANCELED) {
                val code: String
                var message: String? = null
                if (data != null && data.hasExtra("key_error")) {
                    val errorResponseCode = data.getSerializableExtra("key_error") as ErrorResponseCode
                    code = if (errorResponseCode == ErrorResponseCode.INVALID_SESSION) {
                        SessionStatusCode.SESSION_EXPIRED.name
                    } else {
                        SessionStatusCode.UNEXPECTED_ERROR.name
                    }
                    message = errorResponseCode.description
                } else {
                    code = SessionStatusCode.USER_CANCEL.name
                }
                sendError(code, message, SessionTask.FACE.name)
            }
        }
        return true
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        setActivity(binding.activity)
        binding.addActivityResultListener(this)
    }

    override fun onDetachedFromActivityForConfigChanges() {
        TODO("Not yet implemented")
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        TODO("Not yet implemented")
    }

    override fun onDetachedFromActivity() {
        TODO("Not yet implemented")
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        context = null
        activity = null
    }
}
