import 'package:uqudo_flutter_plugin/builder/BackgroundCheckConfiguration.dart';
import 'package:uqudo_flutter_plugin/builder/Document.dart';
import 'package:uqudo_flutter_plugin/builder/FacialRecognitionConfiguration.dart';
import 'package:uqudo_flutter_plugin/builder/Enrollment.dart';
import 'package:uqudo_flutter_plugin/builder/ReadingConfiguration.dart';
import 'package:uqudo_flutter_plugin/builder/AccountRecoveryConfiguration.dart';
export 'package:uqudo_flutter_plugin/uqudo_builder.dart';

enum DocumentType {
  BHR_ID, //Bahrain National Id
  GENERIC_ID, //Generic National Id
  KWT_ID, //Kuwait National Id
  OMN_ID, //Oman National Id
  PAK_ID, //Pakistan National Id
  PASSPORT, //Passport
  SAU_ID, //Saudi Arabia National Id
  UAE_ID, //UAE National Id
  UAE_DL, //UAE Driving license
  UAE_VISA, //UAE visa
  UAE_VL //UAE vehicle license with no Facial recognition supported
}

class EnrollmentBuilder {
  List<Document> documentList;
  String authorizationToken;
  String nonce;
  bool isRootedDeviceAllowed;
  bool isSecuredWindowsDisabled;
  FacialRecognitionConfiguration facialRecognitionSpecification;
  BackgroundCheckConfiguration backgroundCheckConfiguration;
  String sessionId;

  setToken(token) {
    this.authorizationToken = token;
    return this;
  }

  setNonce(nonce) {
    this.nonce = nonce;
    return this;
  }

  setSessionId(sessionId) {
    this.sessionId = sessionId;
    return this;
  }

  enableRootedDeviceUsage() {
    this.isRootedDeviceAllowed = true;
    return this;
  }

  disableSecureWindow() {
    this.isSecuredWindowsDisabled = true;
    return this;
  }

  enableFacialRecognition([FacialRecognitionConfiguration configuration]) {
    if (configuration == null) {
      this.facialRecognitionSpecification =
          new FacialRecognitionConfigurationBuilder().build();
    } else {
      this.facialRecognitionSpecification = configuration;
    }
    return this;
  }

  add(document) {
    if (this.documentList == null) {
      this.documentList = new List();
    }
    this.documentList.add(document);
    return this;
  }

  build() {
    return new Enrollment(
        this.documentList,
        this.authorizationToken,
        this.nonce,
        this.isRootedDeviceAllowed,
        this.isSecuredWindowsDisabled,
        this.facialRecognitionSpecification,
        this.backgroundCheckConfiguration,
        this.sessionId);
  }
}

class FacialRecognitionConfigurationBuilder {
  bool isHelpPageDisabled;
  bool isEnrollFace;
  int scanMinimumMatchLevel;
  int readMinimumMatchLevel;

  disableHelpPage() {
    this.isHelpPageDisabled = true;
    return this;
  }

  enrollFace() {
    this.isEnrollFace = true;
    return this;
  }

  setScanMinimumMatchLevel(scanMinimumMatchLevel) {
    this.scanMinimumMatchLevel = scanMinimumMatchLevel;
    return this;
  }

  setReadMinimumMatchLevel(readMinimumMatchLevel) {
    this.readMinimumMatchLevel = readMinimumMatchLevel;
    return this;
  }

  build() {
    return new FacialRecognitionConfiguration(
        this.isHelpPageDisabled,
        this.isEnrollFace,
        this.scanMinimumMatchLevel,
        this.readMinimumMatchLevel);
  }
}

class BackgroundCheckConfigurationBuilder {
  bool isDisableConsent;

  disableConsent() {
    this.isDisableConsent = true;
    return this;
  }

  build() {
    return new BackgroundCheckConfiguration(this.isDisableConsent);
  }
}

class DocumentBuilder {
  DocumentType documentType;
  ReadingConfiguration readingConfiguration;
  bool isHelpPageVisible;
  int faceScanMinimumMatchLevel;
  int faceReadMinimumMatchLevel;

  setDocumentType(documentType) {
    this.documentType = documentType;
    return this;
  }

  disableHelpPage() {
    this.isHelpPageVisible = false;
    return this;
  }

  enableReading([ReadingConfiguration readingConfiguraiton]) {
    if (readingConfiguraiton == null) {
      this.readingConfiguration = new ReadingConfigurationBuilder().build();
    } else {
      this.readingConfiguration = readingConfiguraiton;
    }
    return this;
  }

  setFaceScanMinimumMatchLevel(faceScanMinimumMatchLevel) {
    this.faceScanMinimumMatchLevel = faceScanMinimumMatchLevel;
    return this;
  }

  setFaceReadMinimumMatchLevel(faceReadMinimumMatchLevel) {
    this.faceReadMinimumMatchLevel = faceReadMinimumMatchLevel;
    return this;
  }

  build() {
    return new Document(
        this.documentType,
        this.readingConfiguration,
        this.isHelpPageVisible,
        this.faceScanMinimumMatchLevel,
        this.faceReadMinimumMatchLevel);
  }
}

class ReadingConfigurationBuilder {
  bool forceReadingValue;
  bool forceReadingIfSupportedValue;

  forceReading(value) {
    this.forceReadingValue = value;
    return this;
  }

  forceReadingIfSupported(value) {
    this.forceReadingIfSupportedValue = value;
    return this;
  }

  build() {
    return new ReadingConfiguration(
        this.forceReadingValue, this.forceReadingIfSupportedValue);
  }
}

class AccountRecoveryConfigurationBuilder {
  String authorizationToken;
  String enrollmentIdentifier;
  String nonce;
  bool isRootedDeviceAllowed;
  bool isSecuredWindowsDisabled;
  bool isHelpPageDisabled;
  int minimumMatchLevel;

  /**
     * Pass the token received from Uqudo to authenticate the SDK
     */
  setToken(token) {
    this.authorizationToken = token;
    return this;
  }

  /**
     * Pass the enrollment identifier for the account to be recovered
     */
  setEnrollmentIdentifier(identifier) {
    this.enrollmentIdentifier = identifier;
    return this;
  }

  /**
     * You can pass your custom nonce to provide security to the enrollment process
     */
  setNonce(nonce) {
    this.nonce = nonce;
    return this;
  }

  /**
     * Whether you want the sdk to run on the rooted devices or not. By default it is false
     */
  enableRootedDeviceUsage() {
    this.isRootedDeviceAllowed = true;
    return this;
  }

  /**
     * To allow user to capture/record screenshot or video of the screen on the device app is installed.
     * Default is screenshot and video recording of the screen is not allowed
     */
  disableSecureWindow() {
    this.isSecuredWindowsDisabled = true;
    return this;
  }

  /**
     * Whether or not to show the help pages to the user. Calling this will disable help page
     * default enabled
     */
  disableHelpPage() {
    this.isHelpPageDisabled = true;
    return this;
  }

  /**
     * Set this to use the value passed for facialRecognition for Account Recovery
     */
  setMinimumMatchLevel(value) {
    this.minimumMatchLevel = value;
    return this;
  }

  /**
     * @returns Intent with the configuration and the token needed to authorize the activity to
     * recover the account
     */
  build() {
    return new AccountRecoveryConfiguration(
        this.authorizationToken,
        this.enrollmentIdentifier,
        this.nonce,
        this.isRootedDeviceAllowed,
        this.isSecuredWindowsDisabled,
        this.isHelpPageDisabled,
        this.minimumMatchLevel);
  }
}
